//
//  KKSideNavigationController.m
//  KKNews
//
//  Created by qiyu on 5/3/13.
//  Copyright (c) 2013 qiyutan. All rights reserved.
//

#import "KKSideNavigationController.h"


#define KKSideBarWidth 260
#define KKSwipeVelocity 1000
#define KKAnimationSecondsPerPoint 0.001

@interface KKSideNavigationController ()<UIGestureRecognizerDelegate>

@end

@implementation KKSideNavigationController
{
    CGFloat _locationX;
    BOOL _startInContentView;
    CGFloat _contentViewX;
    UIPanGestureRecognizer *_panRecognizer;
}

static KKSideNavigationController *s_sideBarNavigationController = nil;
+ (id)sharedController
{
    return s_sideBarNavigationController;
}

- (void)dealloc
{
    if(self == s_sideBarNavigationController){
        s_sideBarNavigationController = nil;
    }
}

- (id)init
{
    NSAssert(!s_sideBarNavigationController, @"only one instance is allowed");
    self = [super init];
    if(self){
        s_sideBarNavigationController = self;
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    if(self.contentViewController){
        [self.view addSubview:self.contentViewController.view];
        [self addChildViewController:self.contentViewController];
    }    
    _panRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(onPan:)];
    _panRecognizer.delegate = self;
    [self.view addGestureRecognizer:_panRecognizer];
}

- (void)onPan:(UIPanGestureRecognizer*)recognizer
{
    CGPoint location = [recognizer locationInView:self.view];
    switch (recognizer.state) {
        case UIGestureRecognizerStateBegan:
        {
            _locationX = location.x;
            _contentViewX = self.contentViewController.view.frame.origin.x;
            CGFloat x = [recognizer locationInView:self.contentViewController.view].x;
            _startInContentView = x > 0 && x < self.contentViewController.view.bounds.size.width;
        }
            break;
        case UIGestureRecognizerStateChanged:
        {
            if(_startInContentView){
                CGFloat x = location.x - _locationX + _contentViewX;
                x = MIN(KKSideBarWidth, x);
                x = MAX(-KKSideBarWidth, x);
                NSLog(@"%f %f %f %f", x, location.x, _locationX, _contentViewX);
                if(x > 0){
                    if(self.leftSideBarController){
                        [self.view insertSubview:self.leftSideBarController.view belowSubview:self.contentViewController.view];
                    }
                    else {
                        x = 0;
                    }
                }
                else if(x < 0){
                    if(self.rightSideBarController){
                        [self.view insertSubview:self.rightSideBarController.view belowSubview:self.contentViewController.view];
                    }
                    else {
                        x = 0;
                    }
                }
                self.contentViewController.view.frame = CGRectMake(x, 0, self.view.bounds.size.width, self.view.bounds.size.height);
            }
        }
            break;
        case UIGestureRecognizerStateEnded:
        {
            CGFloat x = location.x - _locationX + _contentViewX;
            CGFloat vx = [recognizer velocityInView:self.view].x;
            if(vx > KKSwipeVelocity){
                if(_navigationState == KKNavigationStateNormal){
                    self.navigationState = KKNavigationStateLeft;
                }
                else if(_navigationState == KKNavigationStateRight){
                    self.navigationState = KKNavigationStateNormal;
                }
            }
            else if(vx < -KKSwipeVelocity){
                if(_navigationState == KKNavigationStateNormal){
                    self.navigationState = KKNavigationStateRight;
                }
                else if(_navigationState == KKNavigationStateLeft){
                    self.navigationState = KKNavigationStateNormal;
                }
            }
            else if(x > self.view.bounds.size.width / 2){
                self.navigationState = KKNavigationStateLeft;
            }
            else if(x < -self.view.bounds.size.width / 2){
                self.navigationState = KKNavigationStateRight;
            }
            else{
                self.navigationState = KKNavigationStateNormal;
            }
        }
            break;
        case UIGestureRecognizerStateCancelled:
        {
            [UIView animateWithDuration:0.25 animations:^{
                self.contentViewController.view.frame = CGRectMake(_contentViewX, 0, self.view.bounds.size.width, self.view.bounds.size.height);
            }];
        }
            break;
        default:
            self.contentViewController.view.frame = CGRectMake(_contentViewX, 0, self.view.bounds.size.width, self.view.bounds.size.height);
            break;
    }
}

- (void)setNavigationState:(KKNavigationState)navigationState
{
    switch (navigationState) {
        case KKNavigationStateNormal:
        {
            CGFloat distance = fabsf(self.contentViewController.view.frame.origin.x);
            if(distance > 0.0001){
                [UIView animateWithDuration:distance * KKAnimationSecondsPerPoint animations:^{
                    self.contentViewController.view.frame = self.view.bounds;
                }];
            }
        }
            break;
        case KKNavigationStateLeft:
        {
            NSAssert(_navigationState!=KKNavigationStateRight, @"It's weird to swith from right to left directly");
            if(nil == self.leftSideBarController){
                return;
            }
            
            CGFloat distance = fabsf(self.contentViewController.view.frame.origin.x - KKSideBarWidth);
            if(distance > 0.0001){
                [self.rightSideBarController.view removeFromSuperview];
                [self.view insertSubview:self.leftSideBarController.view belowSubview:self.contentViewController.view];
                [UIView animateWithDuration:distance * KKAnimationSecondsPerPoint animations:^{
                    self.contentViewController.view.frame = CGRectMake(KKSideBarWidth, 0, self.view.bounds.size.width, self.view.bounds.size.height);
                }];
            }
            
        }
            break;
        case KKNavigationStateRight:
        {
            NSAssert(_navigationState!=KKNavigationStateLeft, @"It's weird to swith from left to right directly");
            if(nil == self.rightSideBarController){
                return;
            }
            CGFloat distance = fabsf(self.contentViewController.view.frame.origin.x + KKSideBarWidth);
            if(distance > 0.0001){
                [self.leftSideBarController.view removeFromSuperview];
                [self.view insertSubview:self.rightSideBarController.view belowSubview:self.contentViewController.view];
                [UIView animateWithDuration:distance * KKAnimationSecondsPerPoint animations:^{
                    self.contentViewController.view.frame = CGRectMake(-KKSideBarWidth, 0, self.view.bounds.size.width, self.view.bounds.size.height);
                }];
            }
        }
            break;
        default:
            NSAssert(NO, @"Inconsistent state");
            break;
    }
    
    _navigationState = navigationState;
}

- (void)setContentViewController:(UIViewController *)aViewController
{
    if(self.contentViewController == aViewController){
        return;
    }
    [self.contentViewController.view removeFromSuperview];
    [self.contentViewController removeFromParentViewController];
    if(self.contentViewController){
        aViewController.view.frame = self.contentViewController.view.frame;
    }
    _contentViewController = aViewController;
    [self.view addSubview:self.contentViewController.view];
    [self addChildViewController:self.contentViewController];
}

- (void)setContentHidden:(BOOL)contentHidden
{
    if(_contentHidden == contentHidden){
        return;
    }
    _contentHidden = contentHidden;
    if(_contentHidden){
        if(self.navigationState == KKNavigationStateLeft){
            CGFloat distance = fabsf(self.contentViewController.view.frame.origin.x - self.view.bounds.size.width);
            if(distance > 0.0001){
                [UIView animateWithDuration:distance * KKAnimationSecondsPerPoint animations:^{
                    self.contentViewController.view.frame = CGRectMake(self.view.bounds.size.width, 0, self.view.bounds.size.width, self.view.bounds.size.height);
                }];
            }
        }
        else if(self.navigationState == KKNavigationStateRight){
            CGFloat distance = fabsf(self.contentViewController.view.frame.origin.x + self.view.bounds.size.width);
            if(distance > 0.0001){
                [UIView animateWithDuration:distance * KKAnimationSecondsPerPoint animations:^{
                    self.contentViewController.view.frame = CGRectMake(-self.view.bounds.size.width, 0, self.view.bounds.size.width, self.view.bounds.size.height);
                }];
            }
        }
        else {
            NSAssert(NO, @"Inconsistent state");
        }
    }
    else {
        self.navigationState = _navigationState;
    }
}

#pragma mark - UIGestureRecognizerDelegate
- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer
{
    if(gestureRecognizer == _panRecognizer && _contentHidden){
        return NO;
    }
    return YES;
}
@end
